@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.8 2003/10/13 13:31:01 abe Exp $
+ * $Id: dlsof.h,v 1.10 2005/08/08 19:48:33 abe Exp $
  */
 
 
@@ -42,19 +42,36 @@
 #include <dirent.h>
 #include <nlist.h>
 #include <setjmp.h>
-#include <signal.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/conf.h>
 #include <sys/filedesc.h>
 #include <sys/ucred.h>
-#define m_stat	mnt_stat
-#define	KERNEL
+
+#if	DARWINV<800
 #include <sys/mount.h>
-#undef	KERNEL
+#define	m_stat	mnt_stat
+#else	/* DARWINV>=800 */
+#include <sys/mount_internal.h>
+#define	m_stat	mnt_vfsstat
+#endif	/* DARWINV>=800 */
+
+#if	DARWINV<800
+#include <sys/uio.h>
+#include <sys/vnode.h>
+#else	/* DARWINV>=800 */
+#include <sys/vnode.h>
+#define	_SYS_SYSTM_H_
+struct nameidata { int dummy; };	/* to satisfy function  prototypes */
+#include <sys/vnode_internal.h>
+#endif	/* DARWINV>=800 */
+
 #include <rpc/types.h>
+#define	KERNEL_PRIVATE
+#include <sys/socketvar.h>
+#undef	KERNEL_PRIVATE
 #include <sys/protosw.h>
 #include <sys/socket.h>
-#include <sys/socketvar.h>
 #include <sys/un.h>
 #include <sys/unpcb.h>
 
@@ -82,6 +99,7 @@
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <net/route.h>
+#include <netinet6/ipsec.h>
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
 #include <netinet/tcp.h>
@@ -89,8 +107,7 @@
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
-#include <sys/uio.h>
-#include <sys/vnode.h>
+#include <arpa/inet.h>
 #include <net/raw_cb.h>
 #include <sys/domain.h>
 #define	pmap	RPC_pmap
@@ -98,6 +115,13 @@
 #include <rpc/pmap_prot.h>
 #undef	pmap
 
+#include <sys/quota.h>
+#include <sys/event.h>
+
+# if	DARWINV<800
+#include <paths.h>
+#undef	MAXNAMLEN
+#include <ufs/ufs/quota.h>
 #include <paths.h>
 #include <ufs/ufs/quota.h>
 #include <ufs/ufs/inode.h>
@@ -106,53 +130,73 @@
 #include <nfs/nfsproto.h>
 #include <nfs/nfsnode.h>
 
-# if	DARWINV<600
+#  if	DARWINV<600
 #include <hfs/hfs.h>
 #undef	offsetof
 # else	/* DARWINV>=600 */
 #define	KERNEL
 #include <hfs/hfs_cnode.h>
 #undef	KERNEL
-# endif        /* DARWINV<600 */
+#  endif        /* DARWINV<600 */
+# endif	/* DARWINV<800 */
 
+# if	DARWINV<800
 #define	time	t1		/* hack to make dn_times() happy */
 #include <miscfs/devfs/devfsdefs.h>
 #undef	time
+# endif	/* DARWINV<800 */
 
+# if	DARWINV<800
 #define	KERNEL
 #include <miscfs/fdesc/fdesc.h>
 #undef	KERNEL
+# endif	/* DARWINV<800 */
+
+# if	DARWINV<800
 #include <sys/proc.h>
+# else	/* DARWINV>=800 */
+#define	PROC_DEF_ENABLED
+#define	sleep	kernel_sleep
+#include <sys/proc_internal.h>
+#undef	sleep
+# endif	/* DARWINV<800 */
+
 #include <kvm.h>
 #undef	TRUE
 #undef	FALSE
 
+# if	DARWINV<800
 #include <sys/sysctl.h>
+# else	/* DARWINV>=800 */
+#include "/usr/include/sys/sysctl.h"
+# endif	/* DARWINV<800 */
 
-/* kinfo_proc.kp_proc */
-#define	P_COMM		kp_proc.p_comm
-#define	P_PID		kp_proc.p_pid
-#define	P_STAT		kp_proc.p_stat
-#define	P_VMSPACE	kp_proc.p_vmspace
-
-/* kinfo_proc.kp_eproc */
-#define	P_ADDR		kp_eproc.e_paddr
-#define	P_PGID		kp_eproc.e_pgid
-#define	P_PPID		kp_eproc.e_ppid
-
-/* kinfo_proc.kp_eproc->e_paddr */
-#define	P_FD		p_fd
-
-#define	_KERNEL
+# if	DARWINV<800
 #define	KERNEL
 #include <sys/fcntl.h>
 #include <sys/file.h>
-#undef	_KERNEL
 #undef	KERNEL
+# else	/* DARWINV>=800 */
+#include <sys/fcntl.h>
+#include <sys/file_internal.h>
+# endif	/* DARWINV<800 */
 
-struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
+# if	defined(HASKQUEUE)
+#include <sys/eventvar.h>
+# endif	/* defined(HASKQUEUE) */
+
+# if	defined(DTYPE_PSXSEM)
+#define	HASPSXSEM				/* has the POSIX semaphore file
+						 * type */
+# endif	/* defined(DTYPE_PSXSEM) */
+
+# if	defined(DTYPE_PSXSHM)
+#define	HASPSXSHM				/* has the POSIX shared memory
+						 * file type */
+# endif	/* defined(DTYPE_PSXSHM) */
+
+struct vop_advlock_args { int dummy; };	/* to satisfy lf_advlock() prototype */
 #include <sys/lockf.h>
-
 #include <sys/lock.h>
 
 /*
@@ -161,31 +205,41 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
  */
 
 # if	defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP)
-#define MAP_ENTRY_IS_A_MAP	0
+#define	MAP_ENTRY_IS_A_MAP	0
 # endif	/* defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP) */
 
 #undef	B_NEEDCOMMIT
 #include <sys/buf.h>
+#include <sys/signal.h>
+#define	user_sigaltstack	sigaltstack
 #include <sys/user.h>
 
 #define	COMP_P		const void
-#define DEVINCR		1024	/* device table malloc() increment */
+#define	DEVINCR		1024	/* device table malloc() increment */
 #define	DIRTYPE		dirent	/* directory entry type */
 
 typedef	u_long		KA_T;
 
 #define	KMEM		"/dev/kmem"
 #define	LOGINML		MAXLOGNAME
-#define MALLOC_P	void
-#define FREE_P		MALLOC_P
-#define MALLOC_S	size_t
+#define	MALLOC_P	void
+#define	FREE_P		MALLOC_P
+#define	MALLOC_S	size_t
 
-#define N_UNIX	"/mach_kernel"
+#define	N_UNIX	"/mach_kernel"
 
-#define QSORT_P		void
+#define	QSORT_P		void
 #define	READLEN_T	int
-#define STRNCPY_L	size_t
-#define SWAP		"/dev/drum"
+#define	STRNCPY_L	size_t
+#define	SWAP		"/dev/drum"
+
+# if	DARWINV>=800
+#define	SZOFFTYPE	unsigned long long
+					/* size and offset internal storage
+					 * type */
+#define	SZOFFPSPEC	"ll"		/* SZOFFTYPE printf specification
+					 * modifier */
+# endif	/* DARWINV>=800 */
 
 
 /*
@@ -194,11 +248,8 @@ typedef	u_long		KA_T;
 
 struct file * Cfp;
 
-extern kvm_t *Kd;
-
-# if	defined(P_ADDR)
+extern int Kd;				/* KMEM descriptor */
 extern KA_T Kpa;
-# endif	/* defined(P_ADDR) */
 
 struct l_vfs {
 	KA_T addr;			/* kernel address */
@@ -224,7 +275,7 @@ struct mounts {
 					 * (symbolic links resolved) */
         dev_t dev;              	/* directory st_dev */
 	dev_t rdev;			/* directory st_rdev */
-	ino_t inode;			/* directory st_ino */
+	INODETYPE inode;		/* directory st_ino */
 	mode_t mode;			/* directory st_mode */
 	mode_t fs_mode;			/* file system st_mode */
         struct mounts *next;    	/* forward link */
@@ -234,10 +285,6 @@ struct mounts {
 #define	X_NCSIZE	"ncsize"
 #define	NL_NAME		n_name
 
-extern int Np;				/* number of kernel processes */
-
-extern struct kinfo_proc *P;		/* local process table copy */
-
 struct sfile {
 	char *aname;			/* argument file name */
 	char *name;			/* file name (after readlink()) */
@@ -247,12 +294,15 @@ struct sfile {
 	u_short mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
-	ino_t i;			/* inode number */
+	INODETYPE i;			/* inode number */
 	int f;				/* file found flag */
 	struct sfile *next;		/* forward link */
 
 };
 
+#define	XDR_VOID	(const xdrproc_t)xdr_void 
+#define	XDR_PMAPLIST	(const xdrproc_t)xdr_pmaplist
+
 
 /*
  * Definitions for rnmh.c
