@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.31 2003/10/15 11:34:06 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.35 2005/08/08 19:53:24 abe Exp $";
 #endif
 
 
@@ -40,7 +40,7 @@ static char *rcsid = "$Id: dnode.c,v 1.31 2003/10/15 11:34:06 abe Exp $";
 
 
 #if	defined(HASFDESCFS) && HASFDESCFS==1
-_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
+_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 #if	defined(HASPROCFS)
@@ -98,7 +98,7 @@ getmemsz(pid)
 static int
 lkup_dev_tty(dr, ir)
 	dev_t *dr;			/* place to return device number */
-	unsigned long *ir;		/* place to return inode number */
+	INODETYPE *ir;			/* place to return inode number */
 {
 	int i;
 
@@ -119,7 +119,7 @@ lkup_dev_tty_again:
 # endif	/* defined(HASDCACHE) */
 
 		*dr = Devtp[i].rdev;
-		*ir = (unsigned long)Devtp[i].inode;
+		*ir = Devtp[i].inode;
 		return(1);
 	    }
 	}
@@ -188,16 +188,17 @@ process_node(va)
 	struct mfsnode m;
 	struct nfsnode n;
 	enum nodetype {NONODE, CDFSNODE, DOSNODE, EXT2NODE, FDESCNODE, INODE,
-		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE} nty;
+		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE, PTYFSNODE} nty;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
-	unsigned long iso_ino, iso_sz;
+	INODETYPE iso_ino;
 	long iso_nlink;
 	int iso_stat;
+	SZOFFTYPE iso_sz;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
@@ -205,7 +206,7 @@ process_node(va)
 
 # if	HASFDESCFS==1
 	static dev_t f_tty_dev;
-	static unsigned long f_tty_ino;
+	static INODETYPE f_tty_ino;
 	static int f_tty_s = 0;
 # endif	/* HASFDESCFS==1 */
 
@@ -239,7 +240,7 @@ process_node(va)
 #if	defined(HASMSDOSFS)
 	struct denode d;
 	u_long dpb;
-	u_long nn;
+	INODETYPE nn;
 	struct msdosfsmount pm;
 #endif	/* defined(HASMSDOSFS) */
 
@@ -262,6 +263,11 @@ process_node(va)
 	size_t sz;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	struct ptyfsnode pt;
+	struct specinfo si;
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASNULLFS)
 
 process_overlaid_node:
@@ -364,8 +370,7 @@ process_overlaid_node:
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
-	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
-		  &iso_nlink, (SZOFFTYPE *)&iso_sz))
+	    if (read_iso_node(v, &iso_dev, &iso_ino, &iso_nlink, &iso_sz))
 	    {
 		(void) snpf(Namech, Namechl, "can't read iso_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
@@ -435,7 +440,7 @@ process_overlaid_node:
 	 */
 	    if (v->v_type == VDIR && (v->v_flag & VROOT)) {
 		(void) snpf(Namech, Namechl, "%s", _PATH_KERNFS);
-		ksb.st_ino = 2;
+		ksb.st_ino = (ino_t)2;
 		ksb.st_size = DEV_BSIZE;
 		ksbs = 1;
 	    } else if (Namech[0] && statsafely(Namech, &ksb) == 0)
@@ -510,7 +515,6 @@ process_overlaid_node:
 	    goto process_overlaid_node;
 #endif	/* defined(HASNULLFS) */
 
-
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 	    if (!v->v_data
@@ -524,10 +528,27 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if    defined(HASPTYFS)
+	case VT_PTYFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&pt, sizeof(pt))) {
+		(void) snpf(Namech, Namechl, "can't read ptyfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = PTYFSNODE;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASEXT2FS)
 	case VT_EXT2FS:
 #endif	/* defined(HASEXT2FS) */
 
+#if	defined(HASLFS)
+	case VT_LFS:
+#endif	/* defined(HASLFS) */
+
 	case VT_UFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
@@ -546,7 +567,11 @@ process_overlaid_node:
 		&&  !kread((KA_T)i.i_din.e2fs_din, (char *)&ed, sizeof(ed)))
 		    edp = &ed;
 # else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
 		edp = &i.i_din.e2fs_din;
+#  else	/* HASEXT2FS>=2 */
+		edp = &i.i_e2din;
+#  endif	/* HASEXT2FS>=2 */
 # endif	/* defined(HASI_E2FS_PTR) */
 
 	    } else
@@ -693,7 +718,11 @@ process_overlaid_node:
 		    rdevs = 1;
 		}
 # else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
 		rdev = i.i_din.e2fs_din.e2di_rdev;
+#  else	/* HASEXT2FS>=2 */
+		rdev = i.i_e2din.e2di_rdev;
+#  endif	/* HASEXT2FS>=2 */
 		rdevs = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
 
@@ -734,13 +763,18 @@ process_overlaid_node:
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (vfs) {
+
+# if	defined(HASSTATVFS)
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
 		dev = (dev_t)vfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 		devs = 1;
 	    }
 	    break;
 #endif	/* defined(HASKERNFS) */
 
-
 #if	defined(HAS9660FS)
 	case CDFSNODE:
 	    if (iso_stat) {
@@ -753,6 +787,21 @@ process_overlaid_node:
 	case NFSNODE:
 	    dev = NVATTR.va_fsid;
 	    devs = 1;
+
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (v->v_un.vu_specinfo
+	    &&  !kread((KA_T)v->v_un.vu_specinfo, (char *)&si, sizeof(si))) {
+		rdev = si.si_rdev;
+		rdevs = 1;
+	    }
+	    if (vfs) {
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+		devs = 1;
+	    }
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 /*
  * Obtain the inode number.
@@ -765,17 +814,17 @@ process_overlaid_node:
 		dpb = (u_long)(pm.pm_BytesPerSec / sizeof(struct direntry));
 		if (d.de_Attributes & ATTR_DIRECTORY) {
 		    if (d.de_StartCluster == MSDOSFSROOT)
-			nn = (u_long)1;
+			nn = (INODETYPE)1;
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_StartCluster) * dpb);
+			nn = (INODETYPE)(cntobn(&pm, d.de_StartCluster) * dpb);
 		} else {
 		    if (d.de_dirclust == MSDOSFSROOT)
-			nn = (u_long)(roottobn(&pm, 0) * dpb);
+			nn = (INODETYPE)(roottobn(&pm, 0) * dpb);
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_dirclust) * dpb);
-		    nn += (u_long)(d.de_diroffset / sizeof(struct direntry));
+			nn = (INODETYPE)(cntobn(&pm, d.de_dirclust) * dpb);
+		    nn += (INODETYPE)(d.de_diroffset / sizeof(struct direntry));
 		}
-		Lf->inode = (unsigned long)nn;
+		Lf->inode = nn;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -786,14 +835,14 @@ process_overlaid_node:
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-	    Lf->inode = (unsigned long)i.i_number;
+	    Lf->inode = (INODETYPE)i.i_number;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (ksbs) {
-		Lf->inode = (unsigned long)ksb.st_ino;
+		Lf->inode = (INODETYPE)ksb.st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -809,17 +858,30 @@ process_overlaid_node:
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-	    Lf->inode = (unsigned long)NVATTR.va_fileid;
+	    Lf->inode = (INODETYPE)NVATTR.va_fileid;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case PFSNODE:
-	    Lf->inode = (unsigned long)p.pfs_fileno;
+	    Lf->inode = (INODETYPE)p.pfs_fileno;
 	    Lf->inp_ty = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	case PTYFSNODE:
+	    if (pt.ptyfs_type == PTYFSptc) {
+		if (pt.ptyfs_fileno > 0x3fffffff)
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno & 0x3fffffff);
+		else
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno - 1);
+	    } else
+		Lf->inode = (INODETYPE)pt.ptyfs_fileno;
+	    Lf->inp_ty = 1;
+	    break;
+#endif	/* defined(HASPTYFS) */
+
 	}
 
 /*
@@ -895,13 +957,17 @@ process_overlaid_node:
 
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
-		    if (nty == INODE) {
+		    switch (nty) {
+		    case INODE:
 
 #if	defined(HASI_FFS)
+			
 			Lf->sz = (SZOFFTYPE)i.i_ffs_size;
 			Lf->sz_def = 1;
+			break;
 #else	/* !defined(HASI_FFS) */
 # if	defined(HASI_FFS1)
+
 			if (ffs == 1) {
 			    if (u1s) {
 				Lf->sz = (SZOFFTYPE)u1.di_size;
@@ -913,28 +979,30 @@ process_overlaid_node:
 				Lf->sz_def = 1;
 			    }
 			}
+			break;
 # else	/* !defined(HASI_FFS1) */
 			Lf->sz = (SZOFFTYPE)i.i_size;
 			Lf->sz_def = 1;
 # endif	/* defined(HASI_FFS1) */
 #endif	/* defined(HASI_FFS) */
 
-		    }
+			break;
+
 
 #if	defined(HASMSDOSFS)
-		    else if (nty == DOSNODE) {
+		    case DOSNODE:
 			Lf->sz = (SZOFFTYPE)d.de_FileSize;
 			Lf->sz_def = 1;
-		    }
+			break;
 #endif	/* defined(HASMSDOSFS) */
 
-		    else if (nty == MFSNODE) {
+		    case MFSNODE:
 			Lf->sz = (SZOFFTYPE)m.mfs_size;
 			Lf->sz_def = 1;
-		    }
+			break;
 
 #if	defined(HASEXT2FS)
-		    else if (nty == EXT2NODE) {
+		    case EXT2NODE:
 # if	defined(HASI_E2FS_PTR)
 			if (edp) {
 			    Lf->sz = (SZOFFTYPE)edp->e2di_size;
@@ -944,11 +1012,11 @@ process_overlaid_node:
 			Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
 			Lf->sz_def = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
-		    }
+			break;
 #endif	/* defined(HASEXT2FS) */
 
-		}
-		else if ((type == VCHR || type == VBLK) && !Fsize)
+		    }
+		} else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
 		break;
 	    }
@@ -984,43 +1052,42 @@ process_overlaid_node:
 		}
 		break;
 	    case N_REGLR:
-		if (nty == INODE) {
+		switch (nty) {
+		case INODE:
 
+#if	defined(HASEFFNLINK)
+		    Lf->nlink = (long)i.HASEFFNLINK;
+#else	/* !defined(HASEFFNLINK) */
 # if	defined(HASI_FFS)
 		    Lf->nlink = (long)i.i_ffs_nlink;
-		    Lf->nlink_def = 1;
 # else	/* !defined(HASI_FFS) */
 #  if	defined(HASI_FFS1)
 		    if (ffs == 1) {
-			if (u1s) {
+			if (u1s)
 			    Lf->nlink = (long)u1.di_nlink;
-			    Lf->nlink_def = 1;
-			}
 		    } else if (ffs == 2) {
-			if (u2s) {
+			if (u2s)
 			    Lf->nlink = (long)u2.di_nlink;
-			    Lf->nlink_def = 1;
-			}
 		    }
 #  else	/* !defined(HASI_FFS1) */
+
 		    Lf->nlink = (long)i.i_nlink;
-		    Lf->nlink_def = 1;
 #  endif	/* defined(HASI_FFS1) */
 # endif	/* defined(HASI_FFS) */
+#endif	/* defined(HASEFFNLINK) */
 
 		    Lf->nlink_def = 1;
-		}
+		    break;
 
 #if	defined(HASMSDOSFS)
-		else if (nty == DOSNODE) {
+		case DOSNODE:
 		    Lf->nlink = (long)d.de_refcnt;
 		    Lf->nlink_def = 1;
-		}
+		    break;
 #endif	/* defined(HASMSDOSFS) */
 
-
 #if	defined(HASEXT2FS)
-		else if (nty == EXT2NODE) {
+		case EXT2NODE:
 # if	defined(HASI_E2FS_PTR)
 		    if (edp) {
 			Lf->nlink = (long)edp->e2di_nlink;
@@ -1030,9 +1097,12 @@ process_overlaid_node:
 		    Lf->nlink = (long)i.i_e2fs_nlink;
 		    Lf->nlink_def = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
-		}
-#endif	/* defined(HASEXT2FS) */
 
+		    break;
+	
+#endif	/* defined(HASEXT2FS) */
+		
+		}
 		break;
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
@@ -1047,10 +1117,15 @@ process_overlaid_node:
 #if	defined(HASNULLFS)
 /*
  * If there is a saved nullfs vfs pointer, propagate its device number.
- * 
  */
 	if (nvfs) {
+
+# if	defined(HASSTATVFS)
+	    dev = nvfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
 	    dev = nvfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 	    devs = 1;
 	}
 #endif	/* defined(HASNULLFS) */
@@ -1122,7 +1197,8 @@ process_overlaid_node:
  * 	ioctl(fd, TIOCNOTTY) files;
  *	/kern files
  *	memory node files;
- *	/proc files.
+ *	/proc files;
+ *	ptyfs files.
  */
 
 	if (type == VBAD)
@@ -1192,14 +1268,62 @@ process_overlaid_node:
 		(void) snpf(ep, sz, "/%d/notepg", p.pfs_pid);
 		ty = "PGID";
 		break;
+
+# if	defined(Pfd)
+	    case Pfd:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fd", p.pfs_pid);
+		ty = "PFD";
+		break;
+# endif	/* defined(Pfd) */
+
+# if	defined(Pmap)
+	    case Pmap:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/map", p.pfs_pid);
+		ty = "PMAP";
+		break;
+# endif	/* defined(Pmap) */
+
+# if	defined(Pmaps)
+	    case Pmaps:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/maps", p.pfs_pid);
+		ty = "PMPS";
+		break;
+# endif	/* defined(Pmaps) */
+
 	    }
 	    if (ty)
 		(void) snpf(Lf->type, sizeof(Lf->type), ty);
-	    if (Namech[0])
-		enter_nm(Namech);
 	}
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASPTYFS)
+	else if (nty == PTYFSNODE) {
+	    (void) snpf(Namech, Namechl, "%s", Lf->fsdir);
+	    Lf->nlink = 1;
+	    Lf->nlink_def = 1;
+	    switch (pt.ptyfs_type) {
+	    case PTYFSpts:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu", (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSptc:
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%lu (master)",
+		    (unsigned long)pt.ptyfs_pty);
+		break;
+	    case PTYFSroot:
+		Lf->sz = 512;
+		Lf->sz_def = 1;
+		break;
+	    }
+	    if (ty)
+		(void) snpf(Lf->type, sizeof(Lf->type), ty);
+	}
+#endif	/* defined(HASPTYFS) */
+
 #if	defined(HASBLKDEV)
 /*
  * If this is a VBLK file and it's missing an inode number, try to
@@ -1229,7 +1353,7 @@ process_overlaid_node:
 		    if ((pfi->pid && pfi->pid == p.pfs_pid)
 		    
 # if	defined(HASPINODEN)
-		    ||  (Lf->inp_ty == 1 && pfi->inode == Lf->inode)
+		    ||  ((Lf->inp_ty == 1) && (pfi->inode == Lf->inode))
 # endif	/* defined(HASPINODEN) */
 
 		    ) {
@@ -1253,7 +1377,9 @@ process_overlaid_node:
 	    if (Sfile && is_file_named((char *)NULL,
 				       ((type == VCHR) || (type == VBLK)) ? 1
 									  : 0))
+	    {
 		Lf->sf |= SELNM;
+	    }
 	    if (ns)
 		Namech[0] = '\0';
 	}
