@@ -53,7 +53,9 @@ static char copyright[] =
  * Darwin-specific items
  */
 
+# if LT_VERS<800
 #undef	DO_TEST
+# endif	/* LT_VERS<800 */
 #endif	/* defined(LT_DIAL_darwin) */
 
 
